// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/UnstableParticles.hh"

namespace Rivet {


  /// @brief e+e-> n nbar
  class FENICE_1998_I471263 : public Analysis {
  public:

    /// Constructor
    RIVET_DEFAULT_ANALYSIS_CTOR(FENICE_1998_I471263);


    /// @name Analysis methods
    /// @{

    /// Book histograms and initialise projections before the run
    void init() {
      // Initialise and register projections
      declare(FinalState(), "FS");
      declare(UnstableParticles(), "UFS");
      book(_nNeutron, 1,1,1);
      for (const string& en : _nNeutron.binning().edges<0>()) {
        const double eval = stod(en);
        if (isCompatibleWithSqrtS(eval)) {
          _sqs = en; break;
        }
      }
      raiseBeamErrorIf(_sqs.empty());
    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
      const FinalState& fs = apply<FinalState>(event, "FS");
      // total hadronic and muonic cross sections
      map<long,int> nCount;
      int ntotal(0);
      for (const Particle& p : fs.particles()) {
        ++nCount[p.pid()];
        ++ntotal;
      }
      if (ntotal==2 && nCount[2112]==1 && nCount[-2112]==1) _nNeutron->fill(_sqs);
    }


    /// Normalise histograms etc., after the run
    void finalize() {
      scale(_nNeutron, crossSection()/ sumOfWeights() /nanobarn);
    }

    /// @}


    /// @name Histograms
    /// @{
    BinnedHistoPtr<string> _nNeutron;
    string _sqs = "";
    /// @}


  };


  RIVET_DECLARE_PLUGIN(FENICE_1998_I471263);


}
